@@ -4,6 +4,7 @@ import os
 import io
 from s3aads.resources import s3_resource, s3_client
 from s3aads import Copy
+from collections import namedtuple
 
 class Table(object):
 
@@ -20,16 +21,29 @@ class Table(object):
       def get_values():
         keys = self.keys
         return list(set(key.split("/")[column_number] for key in keys))
-
       return get_values
 
+    def filter_generator(self, column_name):
+      def filter_by(val):
+        return [obj for obj in self.objects if getattr(obj, column_name) == val]
+      return filter_by
+
     for i, col in enumerate(self.columns):
       setattr(self, f"{col}s", column_generator(self, i))
+      setattr(self, f"filter_objects_by_{col}", filter_generator(self, col))
 
   @property
-  def keys(self) -> list:
+  def keys(self) -> List:
     return self.query_by_key("")
 
+  @property
+  def objects(self) -> List[object]:
+    result = []
+    for key in self.keys:
+      d = dict(zip(self.columns, key.split("/")))
+      result.append(namedtuple("Object", d.keys())(*d.values()))
+    return result
+
   def first_column_values(self):
     keys = self.keys
     return list(set(key.split("/")[0] for key in keys))
@@ -57,10 +71,11 @@ class Table(object):
     obj.download_fileobj(stringio)
     return stringio.getvalue()
 
-  def insert_by_key(self, key, data):
+  def insert_by_key(self, key, data, **kwargs):
     obj = self.database.bucket.Object(os.path.join(self.name, key))
     return obj.put( 
-      Body=data
+      Body=data,
+      **kwargs
     )
 
   def delete_by_key(self, key):
@@ -94,8 +109,9 @@ class Table(object):
       raise Exception("data must be bytes")
 
     column_placeholder = self.__form_column_placeholder__()
+    metadata = kwargs.pop("metadata", {})
     key = column_placeholder.format(**kwargs)
-    return self.insert_by_key(key, data)
+    return self.insert_by_key(key, data, **metadata)
 
   def insert_string(self, **kwargs):
     if kwargs.get("data") is None:
